/*
 * SPI Programmer Example for FT2232 devices using libftdi
 * (c)2010 Tomek Cedro (http://www.tomek.cedro.info)
 *
 */


#include <stdlib.h>
#include <stdio.h>
#include <signal.h>
#include <unistd.h>
#include <ftdi.h>

int cable_vid=0x0403;
//int cable_pid=0xbbe2; //Kristech KT-LINK
int cable_pid=0xcff8; //Amontec JTAGKey

struct ftdi_context *ftdic;
struct ftdi_device_list devlist;


int list_ftdi_devices(struct ftdi_context *ftdic, struct ftdi_device_list *devlist, int vid);
int ftdiInit(int vid, int pid);
int ftdiDeInit();
int checkForErrors(int result);
int testBitbang();
int testMpsse();
void eject();



//////////////////////////
// MAIN
//////////////////////////

int main(){
 int ret, ret1, ret2, ret3;

 signal(SIGINT, eject); 

 ftdiInit(cable_vid, cable_pid);

// testBitbang(&ftdic);

 testMpsse(&ftdic);
 return 0;
 
 ftdi_disable_bitbang(ftdic);

 return ftdiDeInit(&ftdic);
}


int list_ftdi_devices(struct ftdi_context *ftdic, struct ftdi_device_list *devlist, int vid){
 //DOES NOT WORK YET ;P
 return -1;
 if (vid==0){
  fprintf(stderr, "You must supply VID!");
  return -1;
 } 
 int pid, ret, found;
 printf("Searching for PID: %x", pid);
 for (pid=0; pid<0xfff; pid++){
  if (!pid%10) printf("...%x", pid);
//  if ( (ret=ftdi_usb_find_all(ftdic, devlist, vid, pid)) < 0){
//   fprintf(stderr, "ftdi_usb_find_all failed: %d (%s)\n", ret, ftdi_get_error_string(&ftdic));
//   return EXIT_FAILURE;
//  } else found+=ret; 
 }
 printf("Found %d devices\n", found);
 return found;
}

int ftdiInit(int vid, int pid){
 int ret;
 ftdic = (struct ftdi_context *) calloc(1, sizeof(struct ftdi_context));
 if (ftdi_init(ftdic) < 0){
  fprintf(stderr, "ERROR: ftdi_init failed!\n");
  exit(EXIT_FAILURE);
 }
 if ((ret = ftdi_usb_open(ftdic, cable_vid, cable_pid)) < 0){
  fprintf(stderr, "ERROR %d: unable to open ftdi device VID:0x%04X PID:0x%04X (%s)\n", ret, vid, pid, ftdi_get_error_string(ftdic));
  ftdi_deinit(ftdic);
  exit(EXIT_FAILURE);
 }
 unsigned int chipid;
 ftdi_read_chipid(ftdic, &chipid);
 printf("FTDI chipid: %X\n", chipid);
 return 0;
}

int ftdiDeInit(){
 int ret;
 if ((ret = ftdi_usb_close(ftdic)) < 0){
  fprintf(stderr, "ERROR %d: Unable to close ftdi device (%s)\n", ret, ftdi_get_error_string(ftdic));
  ftdi_deinit(ftdic);
  return EXIT_FAILURE;
 }
 ftdi_deinit(ftdic);
 free(ftdic);
 return EXIT_SUCCESS;
}

int checkForErrors(int result){
 if (result < 0){
  fprintf(stderr, "ERROR: %s\n", ftdi_get_error_string(ftdic));
  exit(result);
 }
}

int ftdiTestBitbang(){
 int ret1, ret2, ret3, x;
 unsigned char buf[2];

 buf[0]=0;
 buf[1]=0xFF;

 //Each interface must have its own context associated!
 struct ftdi_context ftdica=*ftdic, ftdicb=*ftdic;

 ret1=ftdi_set_interface(&ftdica, INTERFACE_A);
 ret2=ftdi_set_bitmode(&ftdica, 0xFF, BITMODE_BITBANG);
 ret3=ftdi_write_data(&ftdica, &buf[0], 1);
 checkForErrors(ret1|ret2|ret3);

 ret1=ftdi_set_interface(&ftdicb, INTERFACE_B);
 ret2=ftdi_set_bitmode(&ftdicb, 0xFF, BITMODE_BITBANG);
 ret3=ftdi_write_data(&ftdicb, &buf[0], 1);
 checkForErrors(ret1|ret2|ret3);

 for (x=0;x<1000;x++){
  ftdi_write_data(&ftdica, &buf[0], 1);
  ftdi_write_data(&ftdicb, &buf[1], 1);
  printf("(A:%02X,B:%02X) ", buf[0], buf[1]);
  printf(x%8==0?"\n":"");
  buf[0]++;
  buf[1]=buf[0]*2;
  usleep(50000);
 }
 return 0;
}


int testMpsse(){
 int ret1, ret2, ret3, x;
 unsigned char buf[16];

 ret1=ftdi_set_interface(ftdic, INTERFACE_ANY);
 ret2=ftdi_set_bitmode(ftdic, 0xFF, BITMODE_MPSSE);
 checkForErrors(ret1|ret2);

 //enable loopback
 buf[0]=0x84;
 ftdi_write_data(ftdic, buf, 0);

 while (getchar()!=' ') {
  buf[0]=0x38;
  buf[1]=4;
  buf[2]='T';
  buf[3]='E';
  buf[4]='S';
  buf[5]='T';
  ftdi_write_data(ftdic, buf, 6);
  printf("TX: ");  for (x=0;x<=10;x++){printf("%X ", buf[x]);}; printf("\n");
  ftdi_read_data(ftdic, buf, 10);
  printf("RX: "); for (x=0;x<=10;x++){printf("%X ", buf[x]);}; printf("\n");
 }

 //disable loopback
 buf[0]=0x85;
 ftdi_write_data(ftdic, buf, 0);

 return 0;
}

void eject(void){
 exit(ftdiDeInit());
}

